#include "mainform.h"
#include "ui_mainform.h"
#include <QTcpSocket>
#include <QTcpServer>
#include <QHostAddress>
#include <QTimer>

/**
 * @brief
 * @note
 * @param
 * @retval
 */
MainForm::MainForm(QWidget* parent) :
    QWidget(parent),
    ui(new Ui::MainForm) {
    ui->setupUi(this);
    connect(ui->btnExit, SIGNAL(clicked()), this, SLOT(close()));
    tcpServer = new QTcpServer(this);
    connect(tcpServer, SIGNAL(newConnection()), this, SLOT(onNewConnection()));
    tcpServer->listen(QHostAddress("192.168.1.11"), 10001);
    qDebug() << "Server is listening";
    qDebug() << "Server Address = " << tcpServer->serverAddress();
    qDebug() << "Server Port    = " << tcpServer->serverPort();
}

/**
 * @brief
 * @note
 * @param
 * @retval
 */
MainForm::~MainForm(void) {
    qDebug() << "Closing application..";
    tcpClient->disconnect();
    tcpServer->close();
    delete ui;
}

/**
 * @brief
 * @note
 * @param
 * @retval
 */
void MainForm::onNewConnection(void) {
    QTextStream out(stdout);

    qDebug() << "Request for new connection received";

    tcpClient = tcpServer->nextPendingConnection();
    connect(tcpClient, SIGNAL(readyRead()), this, SLOT(readData()));
    connect(tcpClient, SIGNAL(disconnected()), tcpClient, SLOT(deleteLater()));
    connect(tcpClient, SIGNAL(disconnected()), this, SLOT(onClientDisconnected()));
    qDebug() << "Client " << tcpClient->peerAddress().toString() << " connected";
}

/**
 * @brief
 * @note
 * @param
 * @retval
 */
void MainForm::readData(void) {
    QByteArray  msg = tcpClient->readAll();
    float       pi = *(float*)msg.data();

    qDebug() << "Received pi = " << pi;

    msg.clear();
    msg.append("Hello");
    tcpClient->write(msg);

}

/**
 * @brief
 * @note
 * @param
 * @retval
 */
void MainForm::onClientDisconnected(void) {
    QTextStream out(stdout);

    qDebug() << "Client " << tcpClient->peerAddress().toString() << " disconnected";
    qDebug() << "----------------------------------\n";
}

/**
 * @brief
 * @note
 * @param
 * @retval
 */
void MainForm::displayError(QAbstractSocket::SocketError socketError) {
    if (socketError == QTcpSocket::RemoteHostClosedError)
        return;

    qDebug() << "Network error";
    qDebug() << "The following error occurred:" << tcpServer->errorString();

    //    tcpClient.close();
    tcpServer->close();
    qDebug() << "Server closed";
}
